#
# mq_install::autostart
#
# @summary The defined type `mq_install::autostart` allows you to create a systemd startup unit for a specified MQ queue manager and
# make sure it is restarted after a system reboot.
# 
# The simples way to use it is:
# 
# ``` puppet
# mq_install::autostart{'QM1':}
# ```
# 
# Here is an example with all possible properties used:
# 
# ``` puppet
# mq_install::autostart{'QM1':
#     path         => '/etc/systemd/system',
#     service_name => 'my_mq,
#     mq_version   => 8,
#     mq_user      => 'mqm_user',
#     mq_group     => 'mqm_group',
#     nofile_limit => 20480,
#     nproc_limit  => 4096,
# }
# ```
#
# @param [Enum['present', 'absent']] ensure
#    Whether you want to ensure or remove the autostart.
#
# @param [Boolean] standby
#    Use `true` if you want to enure the startup in standby mode.
#
# @param title
#    The title is used to specfy the name of the queue manaer you want to ensure autostarted.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#    }
#    ```
#
# @param [String[1]] path
#    Allows you to specify the path where the unit-file will be placed.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      ...
#      path         => '/etc/mysystemd/system',
#      ...
#    }
#    ```
#    The default value is `/etc/systemd/system'.
#
# @param [String[1]] service_name
#    Allows you to specfy the name of the service.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      ...
#      service_name => 'my_mq,
#      ...
#    }
#    ```
#    The default value is : `"mq@$queuemanager_name`.
#
# @param [Integer] mq_version
#    Allows you to specify the MQ versions.
#    At this point in time it is only used for comments in the unit.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      ...
#      mq_version   => 8,
#      ...
#    }
#    ```
#
# @param [String[1]] mq_user
#    Allows you to specify the OS user the queue manage will be started under.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      ...
#      mq_user      => 'mqm_user',
#      ...
#    }
#    ```
#    The default value is `mqm` and is correct in most situations.
#
# @param [String[1]] mq_group
#    Allows you to specify the OS group the queue manage will be started under.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      ...
#      mq_group     => 'mqm_group',
#      ...
#    }
#    ```
#    The default value is `mqm` and is correct in most situations.
#
# @param [Integer] nofile_limit
#    The nofile limit used for the process running the queue manager.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      ...
#      nofile_limit => 20480,
#      ...
#    }
#    ```
#
# @param [Integer] nproc_limit
#    The nproc limit used for the process running the queue manager.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      path         => '/etc/systemd/system',
#      service_name => 'my_mq,
#      mq_version   => 8,
#      mq_user      => 'mqm_user',
#      mq_group     => 'mqm_group',
#      nofile_limit => 20480,
#      nproc_limit  => 4096,
#    }
#    ```
#
# @param [Boolean] multi_instance
#    When `true` the queue manager wll be started in multi instance mode.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      ...
#      multi_instance => true,
#      ...
#    }
#    ```
#
# @param [String[1]] unit_template
#    The name of the epp template to use for the systemd unit file.
#    The default value is: `mq_install/mq.service.epp`.
#    Here is an example on how to use this:
#    ``` puppet
#    mq_install::autostart{'QM1':
#      ...
#      unit_template => 'my_module/my_template.epp',
#      ...
#    }
#    ```
#    The template is called with these parameters:
#    ```
#    'mq_version'     => $mq_version,
#    'mq_user'        => $mq_user,
#    'mq_group'       => $mq_group,
#    'nofile_limit'   => $nofile_limit,
#    'nproc_limit'    => $nproc_limit,
#    'multi_instance' => $multi_instance,
#    'standby'        => $standby,
#    ```
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mq_install::autostart(
  Enum['present', 'absent'] $ensure = 'present',
  String[1] $path                   = lookup('mq_install::autostart::path'),
  String[1] $service_name           = lookup('mq_install::autostart::service_name'),
  Integer   $mq_version             = lookup('mq_install::autostart::mq_version'),
  String[1] $mq_user                = lookup('mq_install::autostart::mq_user'),
  String[1] $mq_group               = lookup('mq_install::autostart::mq_group'),
  Integer   $nofile_limit           = lookup('mq_install::os::nofile'),
  Integer   $nproc_limit            = lookup('mq_install::os::nproc'),
  Boolean   $multi_instance         = lookup('mq_install::autostart::multi_instance'),
  Boolean   $standby                = lookup('mq_install::autostart::standby'),
  String[1] $unit_template          = lookup('mq_install::autostart::unit_template'),
)
{
  mq_install { "mq_install::autostart::${title}": }

  case($ensure) {
    'absent':  {
      $service_ensure =  'stopped'
      $service_enable =  false
    }
    'present': {
      $service_ensure = 'running'
      $service_enable =  true
    }
    default: { fail('Invalid value for ensure found.')}
  }

  if $standby {
    $stop_options = '-x'
  } else {
    $stop_options = '-i'
  }
  $queue_manager_name = $title

  easy_type::debug_evaluation() # Show local variable on extended debug

  case $::kernel {
    'Linux': {
      if $::systemd == undef  { fail 'Module camptocamp-systemd required. Probably not installed'}
      if $::systemd {
        unless defined(Systemd::Unit_file["${service_name}.service"]) {
          ::systemd::unit_file { "${service_name}.service":
            ensure  => $ensure,
            path    => $path,
            content => epp($unit_template, {
              'mq_version'     => $mq_version,
              'mq_user'        => $mq_user,
              'mq_group'       => $mq_group,
              'nofile_limit'   => $nofile_limit,
              'nproc_limit'    => $nproc_limit,
              'multi_instance' => $multi_instance,
              'standby'        => $standby,
              }),
            notify  => Service["${service_name}${queue_manager_name}"],
          }
        }

        #
        # When mq_manager starts the Queue Manager, it does so outside of the systemd context. To make
        # sure we can start it with systemd, we first have to stop it and then start the service.
        #
        exec {"stop ${queue_manager_name} running outside systemd":
          command => "/opt/mqm/bin/endmqm ${stop_options}  ${queue_manager_name}",
          path    => '/bin',
          user    => $mq_user,
          group   => $mq_group,
          onlyif  => "ps -eaf | egrep '.*${queue_manager_name}.*-u ${mq_user}' | grep -v grep",
          unless  => "systemctl --state running | grep ${queue_manager_name}",
          require => Systemd::Unit_file["${service_name}.service"],
        }

        -> service {"${service_name}${queue_manager_name}":
          ensure => $service_ensure,
          enable => $service_enable,
        }
      } else {

        file { "${path}/mq":
          ensure => $ensure,
          source => 'puppet:///modules/mq_install/mq',
          mode   => '0755',
        }
        ~> service { $service_name:
          ensure => $service_ensure,
          enable => $service_enable,
        }
      }
    }
    'SunOS', 'AIX': {
      file { "${path}/mq":
        ensure => $ensure,
        source => 'puppet:///modules/mq_install/mq',
        mode   => '0755',
      }
      ~> service { $service_name:
        ensure   => $service_ensure,
        enable   => $service_enable,
        provider => 'init',
      }
    }
    default: {
      fail "Kernel ${::kernel} is not (yet) supported by the mq_install module."
    }
  }
}
