#
# mq_install::software::unix
#
# @summary This class install's specified version of the the MQ software for Unix like systems on the system.
# This is an internal defined type and should not be used by client code.
#
# @param [String[1]] source_location
#    The location of the MQ software.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          source_location => '/software',
#          ...
#        }
#
# @param [String[1]] service_window
#    The service_window Puppet will use to execute a specified upgrade.
#    When Puppet detects a mismatch between the current version and the version in your manifest, it will start to execute an upgrade. But in
#    general, you want some control over when that happens. You don't want Puppet to stop your queue managers during regular production hours to do the upgrade.
#    The `service_window` parameter allows this control. The parameter can have the following values:
#    - DIRECT
#    - INSTALL_ONLY
#    - A time range
#    ## DIRECT
#    `DIRECT` is the normal behavior of Puppet (e.g., **not** the default of the parameter). When a manifest change is detected, Puppet will immediately execute the upgrade and, in this process, stop the MQ queue managers.
#    ## INSTALL_ONLY
#    This is the default value for this class. In this case, Puppet will use the specified manifest values **ONLY** on an initial install. When Puppet detects MQ is already running, it will leave it like it is and **NOT** do the upgrade. You can use, for example, a bolt plan or task to execute the upgrade at a moment that suits your requirements.
#    ## time range
#    When you have a specific service window wherein Puppet **is** allowed to do the MQ shutdown and execute the upgrade procedure, you can specify a time range. An example of a time range would be: '01:00 - 02:00`
#
# @param [Pattern[/\d+\.\d+\.\d+\.\d/]] version
#    The version to be installed.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          version => '9.0.0.0',
#          ...
#        }
#
# @param [String[1]] tmp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          tmp_dir => '/my_tmp_dir',
#          ...
#        }
#    On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [String[1]] file_name
#    The file containing the install kit for the MQ software.
#
# @param [Array[String[1]]] selected_components
#    The components that the user has selected to be installed or updated.
#
# @param [Variant[Enum['on_failure'], Boolean]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          logoutput => true,
#          ...
#        }
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mq_install::software::unix(
  String[1]                            $source_location,
  Pattern[/\d+\.\d+\.\d+\.\d/]         $version,                              # Version of the MQ software to install
  String[1]                            $file_name,                            # source file to use
  Array[String[1]]                     $selected_components,                  # The selected components
  String[1]                            $service_window,                       # If needed the service window to apply or maybe value 'INSTALL_ONLY' or 'ALWAYS'
  String[1]                            $tmp_dir              = '/tmp',        # Directory used for unpacking
  Variant[Enum['on_failure'], Boolean] $logoutput            = 'on_failure',  # Output of the execs
)
{
  mq_install { 'mq_install::software': }

  if $service_window == 'INSTALL_ONLY' {
    $service_schedule  = undef
  } else {
    $service_schedule  = 'MQ_SOFTWARE_SERVICE_WINDOW'
  }

  $supported_components = lookup('mq_install::software::supported_components', Hash)
  $version_components   = $version.split('\.')
  $package_version      = "${version_components[0]}.${version_components[1]}.${version_components[2]}-${version_components[3]}"


  #
  # If the installed version is the same as the requested version. Do nothing
  #
  $components_to_remove = $::mq_installed_components - $selected_components
  $components_to_add    = $selected_components - $::mq_installed_components

  # On SunOS we need to reinstall the software when a component is
  # removed or added to we need to fetch it then too
  if $::kernel == 'SunOS' {
    $fetch_software = $components_to_remove != [] or $components_to_add != []
  } else {
    $fetch_software = $components_to_add != []
  }


  if $service_window != 'INSTALL_ONLY' {

    echo { "Stopping MQ managers and installing version ${version}":
      withpath => false,
      schedule => $service_schedule,
    }

    #
    # If we have a version change e.g. 9.1 to 9.2 or 8 to 9, we first need to remove the current version. But
    # we only do this when the service_window is anything else as 'INSTALL_ONLY'
    #
    if mq_install::version_upgrade($::mq_version, $version) {
      #
      # MQ only allows us to install a fixpack package when no components
      # are running. So we have to stop all before continuing.
      #
      class { '::mq_install::internal::mq_stop':
        schedule  => $service_schedule,
        logoutput => $logoutput,
      }

      case $::kernel {
        'Linux': {
          $version_to_remove    = $::mq_version[0,3]
          exec {"uninstall RPMS for version ${::mq_version}":
            command   => "rpm -qa | grep MQSeries |grep ${version_to_remove} | xargs yum remove -y",
            path      => '/bin',
            schedule  => $service_schedule,
            require   => Class['::mq_install::internal::mq_stop'],
            logoutput => $logoutput,
          }
        }
        'SunOS': {
          exec {"uninstall MQ ${::mq_version}":
            command   => 'yes | pkgrm mqm',
            path      => ['/usr/sbin', '/bin'],
            schedule  => $service_schedule,
            logoutput => $logoutput,
          }
        }
        default:  {
          fail "Kernel ${::kernel} is not (yet) supported by the mq_install module."
        }
      }
    } elsif $components_to_remove != [] {
      #
      # MQ only allows us to uninstall a package when no components
      # are running. So we have to stop all before continuing.
      #
      class { '::mq_install::internal::mq_stop':
        schedule  => $service_schedule,
        logoutput => $logoutput,
      }
      contain '::mq_install::internal::mq_stop'

      case $::kernel {
        'Linux', 'AIX': {
          $components_to_remove.each |$comp| {
            package { $comp:
              ensure   => 'absent',
              schedule => $service_schedule,
              require  => Class['::mq_install::internal::mq_stop'],
            }
          }
        }
        'SunOS': {
          #
          # SunOS has one pkg package containing all components. The other U*X like
          # OS-es have a package for every component. So if we need to remove a component,
          # we remove everyting and reinstall only the requested componens
          #
          file { "${tmp_dir}/rmadmin":
            ensure   => 'present',
            schedule => $service_schedule,
            content  => 'action=nocheck',
          }
          -> exec {'remove mqm':
            command  => "pkgrm -n -a ${tmp_dir}/rmadmin mqm",
            path     => '/usr/sbin',
            schedule => $service_schedule,
            require  => Class['::mq_install::internal::mq_stop'],
          }
        }
        default: { fail "Internal error. Kernel ${::kernel} not supported."}
      }
    }
  }

  if !($::mq_version != 'not-installed' and $service_window == 'INSTALL_ONLY') {

    if ($fetch_software or mq_install::version_upgrade($::mq_version, $version)) {
      #
      # When the tmp_dir is not defined, define it here.
      #
      unless defined(File[$tmp_dir]) {
        file { $tmp_dir:
          ensure   => 'directory',
          schedule => $service_schedule,
          before   => Archive["${tmp_dir}/${file_name}"],
        }
      }
      archive { "${tmp_dir}/${file_name}":
        ensure       => present,
        extract      => true,
        extract_path => $tmp_dir,
        source       => "${source_location}/${file_name}",
        creates      => "${tmp_dir}/MQServer",
        cleanup      => true,
        schedule     => $service_schedule,
        require      => File[$tmp_dir],
      }
      #
      # On AIX no mqlicense is available
      #
      if $::kernel != 'AIX' {
        exec { 'Accept MQ License Requirements':
          command  => "sudo ${tmp_dir}/MQServer/mqlicense.sh -accept",
          path     => '/usr/bin',
          creates  => "${tmp_dir}/mq_license_${version}/license/status.dat",
          schedule => $service_schedule,
          require  => Archive["${tmp_dir}/${file_name}"],
        }
      }
      case $::kernel {
        'SunOS' : {
          if $fetch_software or mq_install::version_upgrade($::mq_version, $version) {
            #
            # SunOS has one pkg package containing all components. The other U*X like
            # OS-es have a package for every component
            #
            file { "${tmp_dir}/answers":
              ensure   => 'present',
              content  => epp('mq_install/answers.epp', {'selected_components' => $selected_components}),
              schedule => $service_schedule,
            }
            -> file { '/opt/mqm':
              ensure   => 'directory',
              schedule => $service_schedule,
            }
            #
            # We use an exec instead of a package because the package will not do
            # a reinstall if the selected components are changed.
            # 
            -> exec {'install mqm':
              command  => "pkgadd -a ${tmp_dir}/MQServer/silent/admin -r ${tmp_dir}/answers -d ${tmp_dir}/MQServer mqm",
              path     => '/usr/sbin',
              schedule => $service_schedule,
              require  => Exec['Accept MQ License Requirements'],
              before   => File['Remove temporary files'],
            }
          }
        }
        'Linux', 'AIX' : {
          $selected_components.each |$comp| {
            case $::kernel {
              'Linux': {
                $provider     = 'rpm'
                $source       = "${tmp_dir}/MQServer/${comp}-${package_version}.x86_64.rpm"
                $package_name = "${comp}-${package_version}"
              }
              'AIX': {
                if $comp == 'mqm.server.rte' {
                  $source       = "${tmp_dir}/MQServer/mqm.server"
                } else {
                  $source       = "${tmp_dir}/MQServer/${comp}"
                }
                $provider     = 'aix'
                $package_name = $comp
              }
              default: { fail "Internal error. Kernel ${::kernel} not supported."}
            }

            package {$package_name:
              ensure   => 'present',
              provider => $provider,
              source   => $source,
              schedule => $service_schedule,
              before   => File['Remove temporary files'],
            }
          }
        }
        default: { fail "Internal error. Kernel ${::kernel} not supported."}
      }

      file {'Remove temporary files':
        ensure   => 'absent',
        path     => "${tmp_dir}/MQServer",
        recurse  => true,
        schedule => $service_schedule,
        force    => true,
      }
    }
  } else {
    echo {'When a software update is detected, it will be skipped, because service window is set to INSTALL_ONLY':
      withpath => false,
    }
  }
}
