#
# mq_install::software::windows
#
# @summary This class install's specified version of the the MQ software for Windows on the system.
# This is an internal defined type and should not be used by client code.
#
# @param [String] source_location
#    The location of the MQ software.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          source_location => '/software',
#          ...
#        }
#
# @param [Pattern[/\d+\.\d+\.\d+\.\d/]] version
#    The version to be installed.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          version => '9.0.0.0',
#          ...
#        }
#
# @param [String] tmp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          tmp_dir => '/my_tmp_dir',
#          ...
#        }
#    On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [Array[String[1]]] selected_components
#    The components that the user has selected to be installed or updated.
#
# @param [String[1]] service_window
#    The service_window Puppet will use to execute a specified upgrade.
#    When Puppet detects a mismatch between the current version and the version in your manifest, it will start to execute an upgrade. But in
#    general, you want some control over when that happens. You don't want Puppet to stop your queue managers during regular production hours to do the upgrade.
#    The `service_window` parameter allows this control. The parameter can have the following values:
#    - DIRECT
#    - INSTALL_ONLY
#    - A time range
#    ## DIRECT
#    `DIRECT` is the normal behavior of Puppet (e.g., **not** the default of the parameter). When a manifest change is detected, Puppet will immediately execute the upgrade and, in this process, stop the MQ queue managers.
#    ## INSTALL_ONLY
#    This is the default value for this class. In this case, Puppet will use the specified manifest values **ONLY** on an initial install. When Puppet detects MQ is already running, it will leave it like it is and **NOT** do the upgrade. You can use, for example, a bolt plan or task to execute the upgrade at a moment that suits your requirements.
#    ## time range
#    When you have a specific service window wherein Puppet **is** allowed to do the MQ shutdown and execute the upgrade procedure, you can specify a time range. An example of a time range would be: '01:00 - 02:00`
#
# @param [String] file_name
#    The file containing the install kit for the MQ software.
#
# @param [Variant[Enum['on_failure'], Boolean]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          logoutput => true,
#          ...
#        }
#
#
# See the file "LICENSE" for the full license governing this code.
#
define mq_install::software::windows(
  String                               $source_location,
  Pattern[/\d+\.\d+\.\d+\.\d/]         $version,                              # Version of the MQ software to install
  Array[String[1]]                     $selected_components,                  # The selected components
  String[1]                            $service_window,                        # If needed the service window to apply or maybe value 'INSTALL_ONLY' or 'ALWAYS'
  String                               $tmp_dir              = 'C:\\tmp',       # Directory used for unpacking
  String                               $file_name            = 'IBM_MQ_9.1_WINDOWS_TRIAL.zip', # Name of install file
  Variant[Enum['on_failure'], Boolean] $logoutput            = 'on_failure',    # Output of the execs
)
{
  mq_install { 'mq_install::software': }
  $supported_components = lookup('mq_install::software::supported_components', Hash)
  #
  # Setup some variables
  #
  $version_components = $version.split('\.')
  #
  # On the package version the last number is separated by an dash
  #
  $package_version    = "${version_components[0]}.${version_components[1]}.${version_components[2]}-${version_components[3]}"

  $components_to_add    = $selected_components - $::mq_installed_components
  $components_to_remove = $::mq_installed_components - $selected_components

  easy_type::debug_evaluation() # Show local variable on extended debug

  if $components_to_add != [] or $components_to_remove != []{

    unless defined(File[$tmp_dir]) {
      file { $tmp_dir:
        ensure => 'directory',
      }
    }

    archive { "${tmp_dir}\\${file_name}":
      ensure       => present,
      extract      => true,
      extract_path => $tmp_dir,
      source       => "${source_location}\\${file_name}",
      creates      => "${tmp_dir}\\MQServer",
      cleanup      => true,
      require      => File[$tmp_dir],
    }

    ->file {"${tmp_dir}\\MQServer":
      ensure  => 'directory',
    }

    -> file { "${tmp_dir}\\MQServer\\Response.ini":
      ensure  => file,
      content => epp('mq_install/response.ini.epp',{'add' => $components_to_add, 'remove' =>$components_to_remove }),
      backup  => false,
    }
    -> package { 'IBM MQ(Installation1)':
      ensure          => installed,
      source          => "${tmp_dir}\\MQServer\\MSI\\IBM MQ.msi",
      install_options => ["USEINI=\"${tmp_dir}\\MQServer\\Response.ini\"", 'AGREETOLICENSE="yes"','/l*v', "${tmp_dir}\\mqserver_install.log"],
    }

    #
    # Because the mq_config module start's right away, we neede to make sure the IBM MQ task
    #  is up and running before leaving this module.
    #
    ~> exec {'wait for IBM Service manager':
      command     => epp('mq_install/wait_for_service_manager.ps1.epp'),
      refreshonly => true,
      provider    => 'powershell',
      timeout     => 60,
    }

    exec {'Remove temporary MQ files':
      path    => 'c:\\windows\\system32',
      command => "cmd.exe /c \"rmdir /s /q ${tmp_dir}\\MQServer\"",
      require => Package['IBM MQ(Installation1)'],
    }

  }
}
